/*
 * anydata.c  --  AnyDATA DTEV-series WAN support module
 *
 * Copyright 2005, 2006 Lab126, Inc.  All rights reserved.
 *
 */

#include <linux/version.h>
#include <linux/kernel.h>
#include <linux/init.h>
#include <linux/module.h>
#include <linux/moduleparam.h>
#include <linux/time.h>
#include <asm/fcntl.h>
#include <asm/uaccess.h>
#include <asm/arch/system.h>
#include <linux/proc_fs.h>
#include <linux/syscalls.h>
#include <linux/delay.h>
#include <linux/interrupt.h>
#include <asm/irq.h>
#include <asm/arch/fiona.h>
#include <linux/sem.h>
#include <linux/ipc.h>
#include <linux/suspend.h>
#include <asm/arch/fpow.h>
#include <asm/arch/fiona_eventmap.h>
#include <net/anydata.h>


#undef ALLOW_NOTIFY_SCRIPT

#define info(format, arg...) printk("wan: " format, ## arg)

#define ANYDATA_VERSION		"0.8.4"

// Define to handle wan switch locally, undef to pass event up to framework
#undef CONFIG_SYSTEM_SWITCH_CONTROL

#define wan_switch()		((GPLR(g_fiona_hw.wan_switch_gpio) & GPIO_bit(g_fiona_hw.wan_switch_gpio)) ? 1 : 0)

#undef INIT_SERIAL

#define WAN_SEM_KEY		ftok(AD_WAN_NOTIFY_TOK, AD_WAN_NOTIFY_ID)

#ifdef CONFIG_FIONA_PM_WAN
extern struct fpow_component *g_wan_fpow_component_ptr;
int wan_register_with_fpow(struct fpow_component **fpow_dev, void *private);
void wan_unregister_with_fpow(struct fpow_component **fpow_dev);
#endif

static struct proc_dir_entry *proc_wan_parent;
static struct proc_dir_entry *proc_wan_ctrl;
static struct proc_dir_entry *proc_wan_switch;

static wan_status_t wan_status = WAN_OFF;

static pid_t wan_en_thread_pid = 0, wan_tph_thread_pid = 0;
static DECLARE_COMPLETION(wan_en_thread_exited);
static DECLARE_COMPLETION(wan_tph_thread_exited);

static wait_queue_head_t power_event_wq, tph_event_wq;
static int power_event = 0, tph_event = 0;
static unsigned long tph_last_seconds = 0;

// minimum allowed delay between TPH IRQs between notifications
#define WAKE_IRQ_INTERVAL		17


int
set_wan_power(
	wan_status_t new_status)
{
	wan_status_t check_status = new_status == WAN_OFF_KILL ? WAN_OFF : new_status;

	if (wan_status == check_status) {
		//info("NOT powering %s WAN module, already in that state\n", new_status == WAN_ON ? "on" : "off");
		return 0;
	}

	// ignore any spurious WAKE line events during module power sequencing
	tph_last_seconds = get_seconds();

	switch (new_status) {

		case WAN_ON :
			info("powering on WAN module\n");

			// bring wake line high
			GPSR(GPIO81_FIONA_P3_MODULE_WAKE) = GPIO_bit(GPIO81_FIONA_P3_MODULE_WAKE);
			
			// wait 100ms
			msleep(100);
			
			// bring power line high
			GPSR(GPIO24_FIONA_P3_WAN_ON_OFF) = GPIO_bit(GPIO24_FIONA_P3_WAN_ON_OFF);
	
			// wait 100ms
			msleep(100);
	
			// bring wake line low
			GPCR(GPIO81_FIONA_P3_MODULE_WAKE) = GPIO_bit(GPIO81_FIONA_P3_MODULE_WAKE);
	
			// wait one second (enters power up process)
			msleep(1000);
	
			// bring wake line high
			GPSR(GPIO81_FIONA_P3_MODULE_WAKE) = GPIO_bit(GPIO81_FIONA_P3_MODULE_WAKE);
	
			// module is powered up at this point
	
			// make sure to keep WAN powered up during sleep
			fiona_pgsr_set(GPIO24_FIONA_P3_WAN_ON_OFF);
			fiona_pgsr_set(GPIO81_FIONA_P3_MODULE_WAKE);

			break;

		default :
			info("error: unknown power request (%d)\n", new_status);

			new_status = WAN_OFF;
			// (fall through)

		case WAN_OFF :
		case WAN_OFF_KILL :
			info("powering off WAN module (%s deregistration)\n", new_status == WAN_OFF_KILL ? "skipping" : "performing");

			// bring wake line low
			GPCR(GPIO81_FIONA_P3_MODULE_WAKE) = GPIO_bit(GPIO81_FIONA_P3_MODULE_WAKE);

			// sleep three seconds (enters power down process)
			msleep(3000);

			// bring wake line high
			GPSR(GPIO81_FIONA_P3_MODULE_WAKE) = GPIO_bit(GPIO81_FIONA_P3_MODULE_WAKE);

			if (new_status == WAN_OFF) {
				// wait twelve seconds for full network deregistration
				msleep(12000);

			} else {
				// skip network deregistration
				msleep(1000);
			}

			// bring power line low
			GPCR(GPIO24_FIONA_P3_WAN_ON_OFF) = GPIO_bit(GPIO24_FIONA_P3_WAN_ON_OFF);
	
			// wait one second (may vary depending on discharing time of Vcc)
			msleep(1000);
	
			// bring wake line low		
			GPCR(GPIO81_FIONA_P3_MODULE_WAKE) = GPIO_bit(GPIO81_FIONA_P3_MODULE_WAKE);

			// module is powered down at this point

			new_status = WAN_OFF;

			// make sure to keep WAN powered down during sleep
			fiona_pgsr_clear(GPIO24_FIONA_P3_WAN_ON_OFF);
			fiona_pgsr_clear(GPIO81_FIONA_P3_MODULE_WAKE);

			break;

	}

	wan_status = new_status;

	//
	// Return no-error.  Currently, there's no way to get an error here,
	// so we return no error value of 0.
	// In the future, it should error if there is not enough power
	// to put the WAN into the proper mode. -njv
	//
	return 0;
}


static void
wan_en_notify_sem(
	void)
{
	int sem_id;

	// retrieve the notification semaphore
	sem_id = sys_semget(AD_WAN_NOTIFY_SEM_KEY, 1, S_IRUGO | S_IWUGO);
	if (sem_id >= 0) {
		struct sembuf sem_operations[1];

		// signal the semaphore
		sem_operations[0].sem_num = 0;		// which semaphore to use
		sem_operations[0].sem_op = 1;		// add one from the semaphore value
		sem_operations[0].sem_flg = 0;		// set wait flag

		// do the signal	
		sys_semop(sem_id, sem_operations, 1);

	} else {
		// semaphore doesn't exist; ignore as nobody may be waiting on it

	}
}


#ifdef ALLOW_NOTIFY_SCRIPT
static void
wan_en_notify_script(
	int enable)
{
	int err;

	char *envp[] = {
		"HOME=/",
		"TERM=linux",
		"PATH=/sbin:/usr/sbin:/bin:/usr/bin",
		NULL
	};

	char *argv[] = {
		AD_WAN_NOTIFY_SCRIPT,
		enable ? "1" : "0",
		NULL
	};

	if ((err = call_usermodehelper(argv[0], argv, envp, 0)) < 0) {
		info("notify script execution failed (%d)\n", err);
	}
}
#endif


static int
wan_en_power_thread(
	void *unused)
{
	int wan_enabled = 0;

	daemonize("wanend");
	allow_signal(SIGKILL);

	while (1) {
		wait_event_interruptible(power_event_wq, power_event);

#ifdef CONFIG_PM
		// Check for sleep request
		if (current->flags & PF_FREEZE) {
			refrigerator(PF_FREEZE);
			continue;
		}
#endif

		if (signal_pending(current)) {
			break;
		}

		msleep(50);

		power_event = 0;

		wan_enabled = wan_switch();

#ifdef CONFIG_SYSTEM_SWITCH_CONTROL
		if (!wan_enabled && wan_status == WAN_ON) {
			set_wan_power(WAN_OFF);
		}
#endif

		// notify any waiting processes
		wan_en_notify_sem();

#ifdef ALLOW_NOTIFY_SCRIPT
		// execute user-level notify script
		wan_en_notify_script(wan_enabled);
#endif

#ifndef CONFIG_SYSTEM_SWITCH_CONTROL
		handle_fiona_event(wan_enabled ? WAN_SW_ON_EVENTCODE : WAN_SW_OFF_EVENTCODE);
#endif

	}

	complete_and_exit(&wan_en_thread_exited, 0);
}


static void
start_wan_en_power_thread(
	void)
{
	pid_t pid;

	init_waitqueue_head(&power_event_wq);

	pid = kernel_thread(wan_en_power_thread, NULL, CLONE_KERNEL);
	if (pid >= 0) {
		wan_en_thread_pid = pid;

	} else {
		wan_en_thread_pid = 0;

	}
}


static void
stop_wan_en_power_thread(
	void)
{
	int ret;

	if (wan_en_thread_pid > 0) {
		ret = kill_proc(wan_en_thread_pid, SIGKILL, 1);
		if (ret == 0) {
			wait_for_completion(&wan_en_thread_exited);
		}
	}
}


static irqreturn_t
wan_en_irq_handler(
	int irq,
	void *data,
	struct pt_regs *regs)
{
	power_event = 1;

	wake_up(&power_event_wq);

	return IRQ_HANDLED;
}


static int
wan_tph_thread(
	void *unused)
{
	unsigned long tph_cur_seconds, tph_delta;

	daemonize("wantph");
	allow_signal(SIGKILL);

	// ignore any startup TPH interrupts
	tph_last_seconds = get_seconds();

	while (1) {
		wait_event_interruptible(tph_event_wq, tph_event);

#ifdef CONFIG_PM
		// check for sleep request
		if (current->flags & PF_FREEZE) {
			refrigerator(PF_FREEZE);

			// the module can generate wake IRQs on power-up; ignore these
			tph_last_seconds = get_seconds();

			tph_event = 0;
			continue;
		}
#endif

		if (signal_pending(current)) {
			break;
		}

		tph_event = 0;

		info("tph event occurred; ");

		// the module may generate wake IRQs several times during
		// the paging period; filter these extra events out
		tph_cur_seconds = get_seconds();

        //
        // The time may have changed backwards, so instead of returning 0 for that case,
        // we return (WAKE_IRQ_INTERVAL + 1) to allow ring to get through, and will resync the
        // tph_last_seconds variable....  We could just resync the tph_last_seconds variable
        // in the case where we go back in time, but we would risk losing a tph, so better to
        // do an extra check in this corner case than to lose a TPH... - njv 7/12/07

		//tph_delta = tph_last_seconds < tph_cur_seconds ? tph_cur_seconds - tph_last_seconds : 0;

        // Changed to <= to handle case where false ring comes right behind real ring (in same second).
        // Not sure if this is possible, but just-in-case....
		tph_delta = (tph_last_seconds <= tph_cur_seconds) ? (tph_cur_seconds - tph_last_seconds) : (WAKE_IRQ_INTERVAL+1);

		if (tph_delta > WAKE_IRQ_INTERVAL) {
			tph_last_seconds = tph_cur_seconds;

			printk("sending WAN_RING_EVENTCODE notification\n");

			handle_fiona_event(WAN_RING_EVENTCODE);

		} else {
			printk("ignoring (time delta %lds)\n", tph_delta);

		}
	}

	complete_and_exit(&wan_tph_thread_exited, 0);
}


static void
start_wan_tph_thread(
	void)
{
	pid_t pid;

	init_waitqueue_head(&tph_event_wq);

	pid = kernel_thread(wan_tph_thread, NULL, CLONE_KERNEL);
	if (pid >= 0) {
		wan_tph_thread_pid = pid;

	} else {
		wan_tph_thread_pid = 0;

	}
}


static void
stop_wan_tph_thread(
	void)
{
	int ret;

	if (wan_tph_thread_pid > 0) {
		ret = kill_proc(wan_tph_thread_pid, SIGKILL, 1);
		if (ret == 0) {
			wait_for_completion(&wan_tph_thread_exited);
		}
	}
}


static irqreturn_t
wan_tph_irq_handler(
	int irq,
	void *data,
	struct pt_regs *regs)
{
	tph_event = 1;

	wake_up(&tph_event_wq);

	return IRQ_HANDLED;
}


static void
wan_init_irqs(
	void)
{
	int res;

	// set up the IRQ for the WAN_EN switch
	set_irq_type(g_fiona_hw.wan_switch_irq, IRQT_BOTHEDGE);

	res = request_irq(g_fiona_hw.wan_switch_irq, wan_en_irq_handler, SA_INTERRUPT, "wan_en", NULL);
	if (res == 0) {
		//info("switch is using IRQ %d\n", FIONA_P3_WAN_EN_IRQ);

	} else {
		info("error %d requesting IRQ %d\n", res,g_fiona_hw.wan_switch_irq);

	}

	// set up the IRQ for the HOST_WAKE line
	set_irq_type(g_fiona_hw.wake_on_wan_irq, IRQT_RISING);

	res = request_irq(g_fiona_hw.wake_on_wan_irq, wan_tph_irq_handler, SA_INTERRUPT, "wan_tph", NULL);
	if (res == 0) {
		//info("wake line is using IRQ %d\n", FIONA_P3_HOST_WAKE_IRQ);

	} else {
		info("error %d requesting IRQ %d\n", res,g_fiona_hw.wake_on_wan_irq);

	}
}


static void
wan_free_irqs(
	void)
{
	free_irq(g_fiona_hw.wan_switch_irq, NULL);
	free_irq(g_fiona_hw.wake_on_wan_irq, NULL);
}


static void
init_wan(
	void)
{
	// configure WAN module GPIOs
	GPCR(GPIO81_FIONA_P3_MODULE_WAKE) = GPIO_bit(GPIO81_FIONA_P3_MODULE_WAKE);
	GPCR(GPIO24_FIONA_P3_WAN_ON_OFF) = GPIO_bit(GPIO24_FIONA_P3_WAN_ON_OFF);

	pxa_gpio_mode(g_fiona_hw.wake_on_wan_md);
	pxa_gpio_mode(GPIO81_FIONA_P3_MODULE_WAKE_MD);
	pxa_gpio_mode(GPIO24_FIONA_P3_WAN_ON_OFF_MD);
	pxa_gpio_mode(g_fiona_hw.wan_switch_md);

	// clear the FFUART outputs to the module (not used)
	pxa_gpio_mode(GPIO39_FIONA_FFTXD_MD);
	pxa_gpio_mode(GPIO40_FIONA_FFDTR_MD);
	pxa_gpio_mode(GPIO41_FIONA_FFRTS_MD);

	// start the handler threads
	start_wan_en_power_thread();
	start_wan_tph_thread();

	// enable the WAN-related IRQs
	wan_init_irqs();
}


static int
proc_control_write(
	struct file *file,
	const char __user *buf,
	unsigned long count,
	void *data)
{
	char lbuf[16];
	wan_status_t new_wan_status, prev_wan_status = wan_status;

	memset(lbuf, 0, 16);

	if (copy_from_user(lbuf, buf, 1)) {
		return -EFAULT;
	}

	if (lbuf[0] == '2') {
		// blindly cut power to the module (for emergency use only!)
		set_wan_power(WAN_OFF_KILL);

	} else {
		// perform normal on/off power handling
		new_wan_status = lbuf[0] == '0' ? WAN_OFF : WAN_ON;
	
		if ((new_wan_status == WAN_ON && prev_wan_status == WAN_OFF) ||
			(new_wan_status == WAN_OFF && prev_wan_status == WAN_ON)) {
			set_wan_power(new_wan_status);
		}
	}

	return count;
}


static int
proc_control_read(
	char *page,
	char **start,
	off_t off,
	int count,
	int *eof,
	void *data)
{
	int len = sprintf(page, "%d\n", wan_status == WAN_OFF ? 0 : 1);

	*eof = 1;

	return len;
}


static int
proc_switch_read(
	char *page,
	char **start,
	off_t off,
	int count,
	int *eof,
	void *data)
{
	int len = sprintf(page, "%d\n", wan_switch());

	*eof = 1;

	return len;
}


static int __init
anydata_init(
	void)
{
	info("AnyDATA DTEV WAN module driver %s\n", ANYDATA_VERSION);

	// only initialize the WAN for P3.5 or greater
	if (system_rev > 0) {

		proc_wan_parent = create_proc_entry(AD_WAN_PROC_WAN, S_IFDIR | S_IRUGO | S_IXUGO, NULL);
		if (proc_wan_parent) {

			// create the "/proc/wan/control" entry
			proc_wan_ctrl = create_proc_entry(AD_WAN_PROC_CONTROL, S_IWUSR | S_IRUGO, proc_wan_parent);
			if (proc_wan_ctrl) {
				proc_wan_ctrl->data = NULL;
				proc_wan_ctrl->read_proc = proc_control_read;
				proc_wan_ctrl->write_proc = proc_control_write;
			}

			// create the "/proc/wan/switch" entry
			proc_wan_switch = create_proc_entry(AD_WAN_PROC_SWITCH, S_IRUGO, proc_wan_parent);
			if (proc_wan_switch) {
				proc_wan_switch->data = NULL;
				proc_wan_switch->read_proc = proc_switch_read;
				proc_wan_switch->write_proc = NULL;
			}

		}

		// configure module port and GPIO intialization
		init_wan();

#ifdef CONFIG_FIONA_PM_WAN
		// Register WAN driver with Fiona Power Manager,
		// we're ignoring result here as we'll still allow driver to
		// work even if we can't do efficient power management...
		if (wan_register_with_fpow(&g_wan_fpow_component_ptr, NULL)) {
			printk("WAN: Failed to register with fpow !!\n");
		}
#endif
		
	} else {
		info("skipping initialization\n");

	}

	return 0;
}


static void __exit
anydata_exit(
	void)
{
	if (system_rev > 0) {

#ifdef CONFIG_FIONA_PM_WAN
		// Unregister with Fiona Power Management System
		wan_unregister_with_fpow(&g_wan_fpow_component_ptr);
#endif

		wan_free_irqs();

		stop_wan_tph_thread();
		stop_wan_en_power_thread();

		if (proc_wan_parent != NULL) {
			remove_proc_entry(AD_WAN_CONTROL, proc_wan_parent);
			remove_proc_entry(AD_WAN_SWITCH, proc_wan_parent);
			remove_proc_entry(AD_WAN_PROC_WAN, NULL);
			
			proc_wan_parent = NULL;
		}
	}
}


module_init(anydata_init);
module_exit(anydata_exit);

MODULE_DESCRIPTION("AnyDATA DTEV WAN support module");
MODULE_AUTHOR("Lab126");
MODULE_LICENSE("Proprietary");

