/*
 * JFFS2 -- Journalling Flash File System, Version 2.
 *
 * Copyright (C) 2005  Zoltan Sogor <weth@inf.u-szeged.hu>,
 *                     Ferenc Havasi <havasi@inf.u-szeged.hu>,
 *                     Patrik Kluba <pajko@halom.u-szeged.hu>,
 *                     University of Szeged, Hungary
 *
 * For licensing information, see the file 'LICENCE' in this directory.
 *
 * $Id$
 *
 */

#ifndef _CENT_SUM_H
#define _CENT_SUM_H

#include <linux/jffs2.h>

#define JFFS2_CS_VERSION	2
#define CENT_SUM_RETRY_EMPTY 3
#define CENT_SUM_BUFFER_SIZE 128*1024 //kmalloc can't allocate more than 128k

#define JFFS2_INOCACHE_ENTRY_SIZE	(sizeof(struct jffs2_cs_inocache))
#define JFFS2_MIN_INOCACHE_NUM		10
#define JFFS2_JEBINFO_ENTRY_SIZE	(sizeof(struct jffs2_cs_jeb))
#define JFFS2_MIN_JEBINFO_NUM		 5
#define JFFS2_RAWREF_ENTRY_SIZE		(sizeof(struct jffs2_cs_rawref))
#define JFFS2_MIN_RAWREF_NUM		10

#define JFFS2_CS_DURING_MOUNT		 1
#define JFFS2_CS_DURING_UNMOUNT		 2
#define JFFS2_CS_CSBLOCK_VALID		 4
#define JFFS2_CS_MOUNT_OFFSET_PARAM	 8	// for Mount param / Sysfs support

#define IS_DURING_MOUNT (c->cs_struct->cs_flags & JFFS2_CS_DURING_MOUNT)
#define IS_DURING_UMOUNT (c->cs_struct->cs_flags & JFFS2_CS_DURING_UNMOUNT)
#define IS_CSBLOCK_VALID (c->cs_struct->cs_flags & JFFS2_CS_CSBLOCK_VALID)
#define HAS_MOUNT_OFFSET_PARAM (c->cs_struct->cs_flags & JFFS2_CS_MOUNT_OFFSET_PARAM)

#define JFFS2_CS_MODE_NONE		0	// CS is not enabled
#define JFFS2_CS_MODE_FEB		1	// First Erase Block
#define JFFS2_CS_MODE_MP_SYSFS	2	// Mount Parameter / SysFS 

#define DECREASE_FREE_SPACE(x) do { typeof(x) _x = (x); \
		c->free_size -= _x; \
		jeb->free_size -= _x; \
		} while(0)

#define INCREASE_WASTED_SPACE(x) do { typeof(x) _x = (x); \
		c->wasted_size += _x; \
		jeb->wasted_size += _x; \
		} while(0)

struct jffs2_eraseblock;

struct cent_sum_offset {
	struct cent_sum_offset *next;
	uint32_t offset;
	uint32_t length;
};

struct cent_sum {
	struct jffs2_eraseblock *csblock;
	uint32_t csblock_free;
	uint32_t cs_log_offset; // for mount param / sysfs support
	unsigned char cs_flags;
	struct cent_sum_offset *offset_list_tail;
	struct cent_sum_offset *offset_list_head;
	uint32_t offset_list_num;
	void *raw_nii;	// stores raw_node_refs and its next_in_ino
};

struct jffs2_cs_inocache {
	jint32_t ino;
	jint32_t nlink;
	jint32_t state;
} __attribute__((packed));

struct jffs2_cs_jeb {
	jint32_t offset;
	jint32_t unchecked_size;
	jint32_t used_size;
	jint32_t dirty_size;
	jint32_t wasted_size;
	jint32_t erase_count;
} __attribute__((packed));

struct jffs2_cs_rawref {
	jint32_t flash_offset;
	jint32_t totlen;
	jint32_t ino;
} __attribute__((packed));

extern unsigned int centsum_offset; // module parameter (sysfs) stores umount log offset for next mount

#if defined(CONFIG_JFFS2_CS)
#define jffs2_cs_active()	(1)

int jffs2_cs_init(struct jffs2_sb_info *c, void *data);
void jffs2_cs_dump_centsum_data(struct jffs2_sb_info *c);
void jffs2_cs_free(struct jffs2_sb_info *c);
void jffs2_cs_free_lists(struct jffs2_sb_info *c);
void jffs2_cs_reset_sb_info(struct jffs2_sb_info *c);
int jffs2_cs_correct_jebinfo_remount(struct jffs2_sb_info *c);
int jffs2_cs_save_rawrefs_nii(struct jffs2_sb_info *c);
int jffs2_cs_restore_rawrefs_nii(struct jffs2_sb_info *c);

#if defined(CONFIG_JFFS2_CS_FEB)
// FEB mode specific functions, ...

#define jffs2_cs_mode()		(JFFS2_CS_MODE_FEB)

int jffs2_cs_init_cs_block_jeb(struct jffs2_sb_info *c);
int jffs2_cs_write_mount_log_jeb(struct jffs2_sb_info *c);
int jffs2_cs_build_filesystem_jeb(struct jffs2_sb_info *c);

#define jffs2_cs_write_mount_log(c) jffs2_cs_write_mount_log_jeb(c)
#define jffs2_cs_write_umount_log(c) jffs2_cs_write_umount_log_jeb(c)
#define jffs2_cs_build_filesystem(c) jffs2_cs_build_filesystem_jeb(c)
#define jffs2_cs_init_cs_block(c) jffs2_cs_init_cs_block_jeb(c)
#define jffs2_cs_parse_mount_params(a,b) (0)

#elif defined(CONFIG_JFFS2_CS_MP_SYSFS)
// MP_SYSFS mode specific functions, ...

#define jffs2_cs_mode()		(JFFS2_CS_MODE_MP_SYSFS)

int jffs2_cs_write_mount_log_mntsys(struct jffs2_sb_info *c);
int jffs2_cs_build_filesystem_mntsys(struct jffs2_sb_info *c);

#define jffs2_cs_write_mount_log(c) jffs2_cs_write_mount_log_mntsys(c)
#define jffs2_cs_write_umount_log(c) jffs2_cs_write_umount_log_mntsys(c)
#define jffs2_cs_build_filesystem(c) jffs2_cs_build_filesystem_mntsys(c)
#define jffs2_cs_init_cs_block(a)	(0)

#endif	// CONFIG_JFFS2_CS_xyz

#else	// CONFIG_JFFS2_CS not defined 

#define jffs2_cs_active()	(0)
#define jffs2_cs_mode()		(JFFS2_CS_MODE_NONE)
#define jffs2_cs_init(a,b)		(0)
#define jffs2_cs_build_filesystem(c)	(0)
#define jffs2_cs_reset_sb_info(c)
#define jffs2_cs_write_mount_log(c)		(0)
#define jffs2_cs_dump_centsum_data(c)
#define jffs2_cs_save_rawrefs_nii(c)	(0)
#define jffs2_cs_restore_rawrefs_nii(c)
#define jffs2_cs_correct_jebinfo_remount(c)
#define jffs2_cs_free(c)
#define jffs2_cs_init_cs_block(c)	(0)
#endif


#endif // _CENT_SUM_H
