/*
 * e-ink display framebuffer driver for bootloader
 *
 * Copyright (C) 2005-2006 Lab126, Inc.  All rights reserved.
 */

#define FB_CLEAR_PANEL_ID_OVERRIDE  (NULL)
#define FB_DRAW_CENTER              (-1)

#define PANEL_ID_ENV                "panel#"
#define PANEL_ID_STR_SIZE           32

enum splash_screen_type
{
    splash_screen_logo = 0,                 // Logo
    splash_screen_low_battery,              // Low Battery
    splash_screen_no_battery,               // No Battery
    splash_screen_powercycle,               // Power Cycle
    splash_screen_doa,                      // Sad Fiona

    splash_screen_service_menu_all,         // Service Menu:
    splash_screen_service_menu_update,      //  Firmware Update
    splash_screen_service_menu_recover,     //  Firmware Reset
    splash_screen_service_menu_exit,        //  Exit
    
    splash_screen_update_snag,              // Firmware Update
    splash_screen_recover_snag,             // Firmware Reset
    
    num_splash_screens,                     // draw_splash_screen(0..num_splash_screens-1)

    splash_screen_from_memory_overlay,      // Used by the <fb draw> command.
    splash_screen_from_memory_draw,         //
    
    splash_screen_invalid
};
typedef enum splash_screen_type splash_screen_type;

#define FBIO_MIN_SCREEN                 splash_screen_logo
#define FBIO_MAX_SCREEN                 num_splash_screens
#define FBIO_SCREEN_IN_RANGE(s)         \
    ((FBIO_MIN_SCREEN <= (s)) && (FBIO_MAX_SCREEN > (s)))

struct eink_screen_info
{
	int xres,
	    yres,
	    bpp;
	u8  *base;
};
typedef struct eink_screen_info eink_screen_info;

// For use with eink_fb_update_display().
//
#define EINK_FULL_UPDATE_OFF    false
#define EINK_FULL_UPDATE_ON     true

// For use with eink_fb_remove().
//
#define EINK_SOFT_RESET         false   // Software reset occurred, wake part before (re-)init.
#define EINK_HARD_RESET         true    // Hardware reset occurred, only init needed.

#ifdef CONFIG_EINK_APOLLO

extern void eink_fb_get_screen_info(eink_screen_info *screen_info);
extern void eink_fb_update_display(bool update_mode);
extern void eink_fb_clear_screen(void);

extern void eink_fb_remove(bool hard_reset);
extern void eink_fb_boot(void);

extern void draw_splash_screen(splash_screen_type which_screen);

extern void string_to_panel_id(char *string, panel_id_t *panel_id);
extern void panel_id_to_string(panel_id_t *panel_id, char *string);
extern int fb_id(panel_id_t *panel_id);
extern int fb_id_string(char *id);
extern int fb_id_init(void);

#else

#define eink_fb_get_screen_info(i)
#define eink_fb_update_display(m)
#define eink_fb_clear_screen()

#define eink_fb_remove(r)
#define eink_fb_boot()

#define draw_splash_screen(s)

#define string_to_panel_id(s, p)
#define panel_id_to_string(p, s)
#define fb_id(p)
#define fb_id_string(s)
#define fb_id_init()

#endif // CONFIG_EINK_APOLLO
