/*
 * Tool for managing arbitrary binary data.
 *
 * Copyright (C)2006-2007 Lab126, Inc.  All rights reserved.
 */

// Interface
//
#include "../ub_boot_globals.h"
#include "../fb.h"

#include "image_manager.h"

// Data
//
#include "image_picture_logo.h"

#include "image_picture_service_menu_all.h"
#include "image_picture_service_menu_update.h"
#include "image_picture_service_menu_recover.h"
#include "image_picture_service_menu_exit.h"

#include "image_picture_update_snag.h"
#include "image_picture_recover_snag.h"

#ifdef CONFIG_USE_ALERT_PICTURES
#include "image_picture_no_battery.h"
#include "image_picture_low_battery.h"
#include "image_picture_powercycle.h"
#include "image_picture_doa.h"
#endif

// Implementation
//
int uncompress_image(image_type which_image, u8 *image_buffer, u32 *buffer_length, void **image_data)
{
    int result = FAILURE;
    
    if ( image_buffer && image_data && (buffer_length && *buffer_length) )
    {
        u32 image_length = 0;
        u8  *image = NULL;

        switch ( which_image )
        {
            // Picture for logo screen.
            //
            case image_picture_logo:
            
                // Check to see whether there's a user boot screen in RAM.
                //
                image_length = get_user_boot_screen();

                if ( image_length )
                {
                    eink_screen_info screen_info;
                    eink_fb_get_screen_info(&screen_info);
                    
                    image = screen_info.base + (screen_info.xres * screen_info.yres * screen_info.bpp)/8;
                    *image_data = (void *)&user_logo_info;
                }
                else
                {
                    *image_data = (void *)&picture_logo_info;
                    image_length = picture_logo_len;
                    image = picture_logo;
                }
            break;

            // Pictures for service menu.
            //
            case image_picture_service_menu_all:
                *image_data = (void *)&picture_service_menu_all_info;
                image_length = picture_service_menu_all_len;
                image = picture_service_menu_all;
            break;
            
            case image_picture_service_menu_update:
                *image_data = (void *)&picture_service_menu_update_info;
                image_length = picture_service_menu_update_len;
                image = picture_service_menu_update;
            break;
            
            case image_picture_service_menu_recover:
                *image_data = (void *)&picture_service_menu_recover_info;
                image_length = picture_service_menu_recover_len;
                image = picture_service_menu_recover;
            break;
            
            case image_picture_service_menu_exit:
                *image_data = (void *)&picture_service_menu_exit_info;
                image_length = picture_service_menu_exit_len;
                image = picture_service_menu_exit;
            break;

            // Pictures for snag keys.
            //            
            case image_picture_update_snag:
                *image_data = (void *)&picture_update_snag_info;
                image_length = picture_update_snag_len;
                image = picture_update_snag;
            break;
                        
            case image_picture_recover_snag:
                *image_data = (void *)&picture_recover_snag_info;
                image_length = picture_recover_snag_len;
                image = picture_recover_snag;
            break;
                        
            // Misc. pictures.
            //
#ifdef CONFIG_USE_ALERT_PICTURES
            case image_picture_no_battery:
                *image_data = (void *)&picture_no_battery_info;
                image_length = picture_no_battery_len;
                image = picture_no_battery;
            break;
            
            case image_picture_low_battery:
                *image_data = (void *)&picture_low_battery_info;
                image_length = picture_low_battery_len;
                image = picture_low_battery;
            break;
            
            case image_picture_powercycle:
                *image_data = (void *)&picture_powercycle_info;
                image_length = picture_powercycle_len;
                image = picture_powercycle;
            break;
            
            case image_picture_doa:
                *image_data = (void *)&picture_doa_info;
                image_length = picture_doa_len;
                image = picture_doa;
            break;
#endif
            // Unrecognized image.
            //
            default:
            break;
        }
        
        if ( image && image_length )
        {
            result = (SUCCESS == gunzip(image_buffer, *buffer_length, image, &image_length)) ? SUCCESS : FAILURE;
            
            if ( SUCCESS == result )
                *buffer_length = image_length;
            else
                *buffer_length = 0;
        }
    }
    
    return ( result );
}
