/* Copyright (C) 2006 Lab126, Inc.  All rights reserved.
 */

#include <asm/arch/pxa-regs.h>
#include <fiona_ioc.h>

#include "ub_boot_globals.h"

boot_globals_t *get_boot_globals(void)
{
    return ( (boot_globals_t *)BOOT_GLOBALS_START );
}

// Simple adjustment value for the amount of time that the RTC is cleared.
#define RTC_SAVE_ADJUST 1 

void restore_rtc(void)
{
    bg_in_use_t *bd = &get_boot_globals()->globals;

    // Validate the saved value and restore if OK.
    if ((bd->saved_rtc ^ bd->saved_rtc_checksum) == 0xFFFFFFFF) {
        // Restore the RTC.
        RCNR = bd->saved_rtc + RTC_SAVE_ADJUST;

        // Clear the saved value so it won't be restored again.
        bd->saved_rtc = bd->saved_rtc_checksum = 0x00000000;
    }
}

void init_boot_globals(void)
{
    unsigned long warm_restart_flag = get_warm_restart_flag();
    int hard_reset = (RCSR_HWR == (RCSR & RCSR_HWR)) || (BOOT_GLOBALS_WARM_FLAG != warm_restart_flag);

    // Restore the PXA RTC (if possible).
    //
    restore_rtc();

    // If the hardware has been physically reset or our warm-restart flag isn't set,
    // clear the boot globals.
    //
    if ( hard_reset )
    {
        unsigned long *boot_globals = (unsigned long *)get_boot_globals();
        int i, j = BOOT_GLOBALS_SIZE >> 2;
        
        // We know that we're long-aligned and long-sized, so we can
        // clear long-words at a time.
        //
        for ( i = 0; i < j; i++ )
            boot_globals[i] = 0;

        // Initialize op-amp offset value.
        //
        if ( INITIALIZE_OP_AMP_OFFSET_STAGE_1 != get_op_amp_offset() )
            set_op_amp_offset(UNINITIALIZED_OP_AMP_OFFSET);
        
        // Flag that we booted dirty.
        //
        set_dirty_boot_flag(1);
    }
    else
    {
        // Always clear the boot-related keys.
        //
        clear_boot_keys();
        
        // Flag that we didn't boot dirty.
        //
        set_dirty_boot_flag(0);

        // Check to see whether we've been asked to go into software update
        // mode programmatically.
        //
        if ( SOFTWARE_UPDATE() )
        {
            // Force us into software update mode using the recovery bootloader.
            //
            switch ( get_update_flag() )
            {
                case BOOT_GLOBALS_UPDATE_FLAG:
                case BOOT_GLOBALS_REPART_FLAG:
                    set_boot_key(CONFIG_SW_UPDATE_KEY);
                break;
                
                case BOOT_GLOBALS_RESTOR_FLAG:
                    set_boot_key(CONFIG_FW_RESET_KEY);
                break;
                
                case BOOT_GLOBALS_SERVIC_FLAG:
                    set_boot_key(CONFIG_MENU_KEY);
                break;
            }

            set_boot_recover(1);
        }
    }
    
    // Assume that we'll be going into recovery mode manually.
    // (If not, load_bootloader() will update the status.)
    //
    set_bootloader_status(BOOTLOADER_STATUS_PASS);
    
    // We're not warm until we've handed things off to the kernel.
    //
    set_warm_restart_flag(BOOT_GLOBALS_COLD_FLAG);
}

void reboot_boot_globals(void)
{
    // Say we're "warm" and not (still) hardware-reset.
    //
    set_warm_restart_flag(BOOT_GLOBALS_WARM_FLAG);
    RCSR = RCSR_HWR;
    
    // Say that the framework is not running.
    //
    set_framework_started(0);
    set_framework_running(0);
}

void clear_boot_keys(void)
{
    set_boot_recover(0);
    set_boot_ioc_key(0);
    set_boot_key(0);
}

// bootloader_status

void set_bootloader_status(unsigned long bootloader_status)
{
    get_boot_globals()->globals.bootloader_status = bootloader_status;
}

unsigned long get_bootloader_status(void)
{
    return get_boot_globals()->globals.bootloader_status;
}

// bootloader_messsage

void set_bootloader_messsage(unsigned long bootloader_messsage)
{
    get_boot_globals()->globals.bootloader_messsage = bootloader_messsage;
}

unsigned long get_bootloader_messsage(void)
{
    return get_boot_globals()->globals.bootloader_messsage;
}

// bootblock_version

void set_bootblock_version(unsigned long bootblock_version)
{
    get_boot_globals()->globals.bootblock_version = bootblock_version;
}

unsigned long get_bootblock_version(void)
{
    return get_boot_globals()->globals.bootblock_version;
}

// build_rev

void set_build_rev(char *build_rev)
{
    if ( build_rev && (BUILD_REV_SIZE >= strlen(build_rev)) )
       strcpy(get_boot_globals()->globals.build_rev, build_rev);
}

char *get_build_rev(void)
{
    return get_boot_globals()->globals.build_rev;
}

// boot_ioc_key

void set_boot_ioc_key(int boot_ioc_key)
{
    get_boot_globals()->globals.boot_ioc_key = boot_ioc_key;
}

int get_boot_ioc_key(void)
{
    return get_boot_globals()->globals.boot_ioc_key;
}

// boot_key

void set_boot_key(int boot_key)
{
    get_boot_globals()->globals.boot_key = boot_key;
}

int get_boot_key(void)
{
    return get_boot_globals()->globals.boot_key;
}

// boot_recover

void set_boot_recover(int boot_recover)
{
    get_boot_globals()->globals.boot_recover = boot_recover;
}

int get_boot_recover(void)
{
    return get_boot_globals()->globals.boot_recover;
}

// serial_num

void set_serial_num(char *serial_num)
{
    if ( serial_num && (SERIAL_NUM_SIZE >= strlen(serial_num)) )
       strcpy(get_boot_globals()->globals.serial_num, serial_num);
}

char *get_serial_num(void)
{
    char *serial_num = get_boot_globals()->globals.serial_num;
    
    if ( 0 == serial_num[0] )
        serial_num = NULL;
    
    return serial_num;
}

#include "common_source_island/boot_globals.c"
