/* 
 * Copyright (C) 2006 Nick Vaccaro, Lab126, Inc.	<nvaccaro@lab126.com>
 *
 * Copyright (C) 2004 Gumstix, Inc.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 *
 * Written by Craig Hughes <craig@gumstix.com> 2004
 *
 */

#ifndef __CONFIG_FIONA_DVT1_H__
#define __CONFIG_FIONA_DVT1_H__

/***********************************************************************
 * This file contains board-specific constants for the following build:
 *
 * Fiona DVT1
 *
 ***********************************************************************/

/*
 * DVT1's run at 400 MHz...
 */
#define CFG_CPUSPEED		0x161

/*
 * But.... DVT1's can run at 200 MHz....
 */
//#define CFG_CPUSPEED		0x141

/*
 * Memory controller constants
 */
// CS0 - NOR
// CS1 - One Nand
// CS2 - One Nand
// CS3 - USB
// CS4 - N/A (ethernet)
// CS5 - Platform Config Bits

#define CFG_MSC0_VAL        0xA278A278
#define CFG_MSC1_VAL        0x22347FF8
#define CFG_MSC2_VAL        0x7FF08008


#if 1
#define CFG_GPDR0_VAL ( 0x0 | \
                BITSET(6)  | BITSET(15) | \
                BITSET(24) | BITSET(25) | BITSET(26) | BITSET(27) | \
                BITSET(28) | BITSET(30) | BITSET(31) )

#else
// Use this definition if using GPIO5 as a debug signal
#define CFG_GPDR0_VAL ( 0x0 | \
                BITSET(5) | \
                BITSET(6)  | BITSET(15) | \
                BITSET(24) | BITSET(25) | BITSET(26) | BITSET(27) | \
                BITSET(28) | BITSET(30) | BITSET(31) )
#endif


#define CFG_GPDR1_VAL ( 0x0 | \
                BITSET(32) | BITSET(33) | BITSET(39) | BITSET(40) | \
                BITSET(41) | BITSET(43) | BITSET(47) | BITSET(48) | \
                BITSET(49) | BITSET(51) | BITSET(52) | BITSET(53) | \
                BITSET(55) | BITSET(56) | BITSET(57) | BITSET(58) | \
                BITSET(59) | BITSET(60) | BITSET(61) | BITSET(62) | \
                BITSET(63) )

#define CFG_GPDR2_VAL ( 0x0 | \
                BITSET(64) | BITSET(65) | BITSET(66) | BITSET(67) | \
                BITSET(68) | BITSET(69) | BITSET(70) | BITSET(78) | \
                BITSET(79) | BITSET(80) | BITSET(81) | BITSET(82) )


#define CFG_GPSR0_VAL       0xFFFFFFFF
#define CFG_GPSR1_VAL       0xFFFFFFFF
#define CFG_GPSR2_VAL       0x001FFFFF

#define CFG_GPCR0_VAL ( 0x0 | \
                BITSET(24) )

#define CFG_GPCR1_VAL ( 0x0 | \
                BITSET(39) | BITSET(40) | BITSET(41) | BITSET(48) | \
                BITSET(51) | BITSET(52) | BITSET(58) | BITSET(59) | \
                BITSET(60) | BITSET(61) | BITSET(62) | BITSET(63) )
                
#define CFG_GPCR2_VAL ( 0x0 | \
                BITSET(64) | BITSET(65) | BITSET(66) | BITSET(67) | \
                BITSET(68) | BITSET(69) | BITSET(70) | BITSET(75) | \
                BITSET(76) | BITSET(77) | BITSET(80) | BITSET(81) | \
                BITSET(82) )

#define CFG_GAFR0_L_VAL	 ( 0x0 | AFSET( 6, 1) | AFSET(15, 2) )

#define CFG_GAFR0_U_VAL	 ( 0x0 | \
                AFSET(16, 2) | AFSET(18, 1) | AFSET(19, 1) | AFSET(20, 1) | \
                AFSET(28, 1) | AFSET(29, 2) | AFSET(30, 1) | AFSET(31, 1) )

#define CFG_GAFR1_L_VAL	 ( 0x0 | \
                AFSET(32, 1) | AFSET(33, 2) | AFSET(34, 1) | AFSET(35, 1) | \
                AFSET(36, 1) | AFSET(37, 1) | AFSET(38, 1) | AFSET(42, 1) | \
                AFSET(43, 2) | AFSET(46, 2) | AFSET(47, 1) )

#define CFG_GAFR1_U_VAL	 ( 0x0 | \
                AFSET(49, 2) )

#define CFG_GAFR2_L_VAL	 ( 0X0 | AFSET(78, 2) | AFSET(79,2) )

#define CFG_GAFR2_U_VAL	 ( 0x0 )

/*
 * Define the desired states of GPIO's during sleep
 * NOTE: This is duplicated in kernel tree: linux/include/asm/arch/fiona.h
 */
#define CFG_PGSR0_FIONA     0xFEFFFEBF

// Use the following if using GPIO5 for debug
// #define CFG_PGSR0_FIONA     0xFEFFFFDF

#define CFG_PGSR1_FIONA     0x01EFFC7F
#define CFG_PGSR2_FIONA     0x001AFF80

// Wake sources
// On/Off Switch (0)
// IOC Wake (1)
// USB_HC   (2)
// USB_DC   (3)
// WAN Wake (4)
// WAN Enable Switch (14)
#define WAKE_PEDR_FIONA     (   0x0  |  \
                            (1 << 0) |  \
                            (1 << 1) |  \
                            (1 << 3) |  \
                            (1 << 4) |  \
                            (1 << 14))

/*************************************************************************
 * GPIO Constants
 ************************************************************************/

/* We clear the PH bit in CFG_PSSR_VAL below to handle the case where we go into
 * "fake sleep" when we see the power switch off in bootloader.  Since PXA
 * actually woke from sleep (but we cleared the sleep bit in the code to
 * indicate such since it wasn't a sleep to resume from - i.e. we want to treat
 * it like a cold boot instead).
 */
#define CFG_PSSR_VAL                0x30
#define CFG_CKEN_VAL                (CKEN14_I2C | CKEN12_MMC | CKEN8_I2S | \
                                     CKEN7_BTUART | CKEN6_FFUART | CKEN5_STUART | \
                                     /*CKEN4_HWUART*/ (1<<4) | CKEN3_SSP | CKEN2_AC97 )

// Define NEEDS_TO_ENABLE_TEMP_SENSOR when GPIOs are needed to enable
// the teperature sensor for the Apollo Eink controller.
//
#define NEEDS_TO_ENABLE_TEMP_SENSOR

// Define HAS_TRUE_POWER_SWITCH when the device has a true switch (as opposed to
// a spring-loaded latch).
//
#define HAS_TRUE_POWER_SWITCH

// Define REQUIRE_BATTERY_INSTALLED to not allow unit to run if 
// battery is not installed.
//
#define REQUIRE_BATTERY_INSTALLED 

#define GPIO_FIONA_DVT1_POWER_SWITCH            0
#define FIONA_DVT1_POWER_SWITCH_ON_POLARITY     0
#define FIONA_DVT1_POWER_SWITCH_OFF_POLARITY    1
#define GPIO_FIONA_DVT1_POWER_SWITCH_MD         (GPIO_FIONA_DVT1_POWER_SWITCH | GPIO_IN)

#define GPIO_FIONA_DVT1_WAN_SWITCH              14
#define FIONA_DVT1_WAN_SWITCH_OFF_POLARITY      0

// Define the battery watermarks device-specifically.
//
#define FIONA_WATERMARK_UPDT                   3720            // Above this in mV, software update.
#define FIONA_WATERMARK_WAN                    3700            // WAN isn't on in bootloader.
#define FIONA_WATERMARK_LOW                    3680            // Above this in mV, boot.
#define FIONA_WATERMARK_CRIT                   3600            // Above this in mV, alert.

#endif  /* __CONFIG_FIONA_DVT1_H__ */

/* vim: set ts=4 sw=4 tw=80: */
