#!/usr/bin/python
#
# pad.py filename buildrev: pads filename to a multiple of 128K,
#   tacks buildrev on to the end, and then checksums the whole
#   thing.

# Import sys for usage.
#
import sys

if len(sys.argv) < 3:
    print "### Usage: pad.py filename buildrev"
    sys.exit(1)

# Import zlib to compute checksum.
#
import zlib

# The flash layout assumes that a bootloader image is exactly 256KB.
# So, we ensure that the bootloader image plus the 252-byte buildrev
# and the 4-byte checksum is no larger than 256KB.
#
k_128K         = 128*1024
k_128K_pad     = k_128K  * '\xFF'

k_meta         = 256
k_meta_pad     = k_meta  * '\xFF'

k_rev          = k_meta  - 4

filesize_limit = (k_128K * 2) - k_meta

# Open filename for reading its data in and printing out how many
# bytes remain.
#
filename = sys.argv[1]
buildrev = sys.argv[2]

print "buildrev =", buildrev

infile = file(filename,'rb')
data = infile.read()
infile.close()

remaining_space = filesize_limit - len(data)
print "filesize = %d, remaining space = %d" % (len(data), remaining_space)

if remaining_space < 0:
    print "### ERROR: Bootloader image is too big to fit in 256KB!"
    sys.exit(2)

# Pad data with 0xFFs to next 128K boundary, then add in the 252-byte
# buildrev and the 4-byte checksum.

# (1) Pad to next 128K.
#
data  = data + k_128K_pad
data  = data[0:(len(data) & ~(k_128K-1))]

# (2) Add in buildrev.
#
fill_rev = buildrev[0:k_rev-1] + '\0'

fill_rev = fill_rev + k_meta_pad
fill_rev = fill_rev[0:(len(fill_rev) & ~(k_meta-1))]

data = data[0:-k_meta] + fill_rev

# (3) Add in checksum.
#
chksm = zlib.adler32(data[0:-4])
print "checksum =", hex(chksm)				
data  = data[0:-4] + "%c" % chr(chksm & 0xFF)
data += "%c" % chr((chksm >>  8) & 0xFF)
data += "%c" % chr((chksm >> 16) & 0xFF)
data += "%c" % chr((chksm >> 24) & 0xFF)

# Write out the padded/buildrev'd/checksummed version of the file and close it.
#
outfile = file(filename,'wb')

outfile.write(data)
outfile.close()
